import { IntlMessageFormat } from 'intl-messageformat';
import { SvelteMap } from 'svelte/reactivity';
import { namedFormat } from './string.js';
export class LocalizationService {
    #config;
    #locales = $state([]);
    #activeLocale = $state(undefined);
    // Loaded localizations: locale -> key -> value, flattened and expanded
    #localizations = new SvelteMap();
    // Which localizations have alread been loaded: locale -> loader -> boolean
    #executedLoaders = new Map();
    #logger;
    #lastLoadedPathname = undefined;
    constructor(config) {
        this.#config = config;
        this.#locales = config.locales ?? [];
        this.#logger = config.logger;
        this.#activeLocale = config.activeLocale;
    }
    get locales() {
        return this.#locales;
    }
    getActiveLocale = () => {
        return this.#activeLocale;
    };
    setActiveLocale = (locale) => {
        if (this.#activeLocale === locale)
            return;
        this.#activeLocale = locale;
        if (!this.#locales.includes(locale)) {
            this.#locales.push(locale);
        }
        if (this.#lastLoadedPathname) {
            this.#logger?.debug(`Reloading localizations for locale ${locale} and pathname ${this.#lastLoadedPathname}`);
            this.loadLocalizations(this.#lastLoadedPathname);
        }
    };
    text = (key, params) => {
        if (!this.#activeLocale)
            return key;
        const text = this.#localizations.get(this.#activeLocale)?.get(key);
        if (!text) {
            this.#logger?.debug(`Translation not found for key neither in currently active locale ${this.#activeLocale} of ${this.#locales}: ${key}`);
            return key;
        }
        return text(params);
    };
    getPSText = (prefix, suffix) => {
        return (key, params) => {
            const prefixKey = prefix ? `${prefix}.${key}` : key;
            const suffixKey = suffix ? `${prefixKey}.${suffix}` : prefixKey;
            return this.text(suffixKey, params);
        };
    };
    loadLocalizations = async (pathname) => {
        if (pathname === '')
            throw new Error('Pathname must not be empty');
        // Do not react to locale changes in this method
        const loadLocales = this.#activeLocale ? [this.#activeLocale] : this.#locales;
        // Get relevant loaders
        const loaders = this.getRelevantLoaders(pathname);
        for (const loader of loaders) {
            for (const loadLocale of loadLocales) {
                if (this.#executedLoaders.get(loadLocale)?.has(loader))
                    continue;
                let localeExecutedLoaders = this.#executedLoaders.get(loadLocale);
                if (!localeExecutedLoaders) {
                    localeExecutedLoaders = new Set([loader]);
                    this.#executedLoaders.set(loadLocale, localeExecutedLoaders);
                }
                else {
                    localeExecutedLoaders.add(loader);
                }
                try {
                    const data = await loader.load(loadLocale);
                    if (!data) {
                        this.#logger?.debug(`No data loaded for locale ${loadLocale} and loader ${loader.key} [${loader.routes ?? '/*'}]`);
                        continue;
                    }
                    else if (this.#activeLocale === undefined) {
                        this.setActiveLocale(loadLocale);
                        // Remove all locales and add the active one
                        loadLocales.splice(0, loadLocales.length, loadLocale);
                    }
                    const flatStringData = this.flattenLocalizationWithPrefix(loader.key, data);
                    let localeLocalization = this.#localizations.get(loadLocale);
                    if (!localeLocalization) {
                        localeLocalization = new SvelteMap();
                        this.#localizations.set(loadLocale, localeLocalization);
                    }
                    const prepare = loader.prepare ?? this.#config.prepare ?? prepareNamedFormat;
                    for (const [key, value] of flatStringData) {
                        localeLocalization.set(key, prepare(loadLocale, value));
                    }
                    // Successfully loaded, no need to try other locales
                    break;
                }
                catch (error) {
                    this.#logger?.debug(`Failed to load localization for locale ${loadLocale} and loader ${loader.key}/${loader.routes ?? 'all'}: ${error}`);
                }
            }
            this.#lastLoadedPathname = pathname;
        }
    };
    getRelevantLoaders(pathname) {
        return (this.#config.loaders?.filter((loader) => {
            if (!loader.routes || loader.routes.length === 0)
                return true;
            return loader.routes?.some((route) => {
                if (typeof route === 'string') {
                    return pathname.startsWith(route);
                }
                return route.test(pathname);
            });
        }) || []);
    }
    flattenLocalizationWithPrefix(prefixKey, localization) {
        const flatLocalization = this.flattenLocalization(localization);
        const prefix = prefixKey || prefixKey == '' ? `${prefixKey}.` : '';
        const result = new Map();
        for (const [key, value] of flatLocalization) {
            result.set(`${prefix}${key}`, value);
        }
        return result;
    }
    flattenLocalization(localization) {
        const result = new Map();
        for (const key in localization) {
            const value = localization[key];
            if (typeof value === 'object') {
                const nested = this.flattenLocalization(value);
                for (const [nestedKey, nestedValue] of nested) {
                    result.set(key + '.' + nestedKey, nestedValue);
                }
            }
            else {
                result.set(key, value);
            }
        }
        return result;
    }
}
export const prepareNamedFormat = (_, value) => {
    return function (params) {
        return namedFormat(value, params);
    };
};
export const prepareICUFormat = (locale, value) => {
    const msg = new IntlMessageFormat(value, locale);
    return function (params) {
        return msg.format(params);
    };
};
